"use strict";
/**
 * @module parjs/combinators
 */
/** */
Object.defineProperty(exports, "__esModule", { value: true });
const issues_1 = require("../issues");
const result_1 = require("../result");
const combinator_1 = require("./combinator");
const parser_1 = require("../parser");
const scalar_converter_1 = require("../scalar-converter");
function manyTill(till, tillOptional) {
    let tillResolved = scalar_converter_1.ScalarConverter.convert(till);
    return combinator_1.defineCombinator(source => {
        return new class ManyTill extends parser_1.ParjserBase {
            constructor() {
                super(...arguments);
                this.type = "manyTill";
                this.expecting = `${source.expecting} or ${tillResolved.expecting}`;
            }
            _apply(ps) {
                let { position } = ps;
                let arr = [];
                let successes = 0;
                while (true) {
                    tillResolved.apply(ps);
                    if (ps.isOk) {
                        break;
                    }
                    else if (ps.atLeast(result_1.ResultKind.HardFail)) {
                        // if till failed hard/fatally, we return the fail result.
                        return;
                    }
                    // backtrack to before till failed.
                    ps.position = position;
                    source.apply(ps);
                    if (ps.isOk) {
                        arr.push(ps.value);
                    }
                    else if (ps.isSoft) {
                        // many failed softly before till...
                        if (!tillOptional) {
                            // if we parsed at least one element, we fail hard.
                            ps.kind = successes === 0 ? result_1.ResultKind.SoftFail : result_1.ResultKind.HardFail;
                            return;
                        }
                        else {
                            // till was optional, so many failing softly is OK.
                            break;
                        }
                    }
                    else {
                        // many failed hard/fatal
                        return;
                    }
                    if (ps.position === position) {
                        issues_1.Issues.guardAgainstInfiniteLoop("manyTill");
                    }
                    position = ps.position;
                    successes++;
                }
                ps.value = arr;
                ps.kind = result_1.ResultKind.Ok;
            }
        }();
    });
}
exports.manyTill = manyTill;

//# sourceMappingURL=many-till.js.map